/*
 * 11_ADC.cpp
 * Created: 2021
 *
 * Pindescription Arduino Nano
 * No.	Func.	PIN-MC	Used as In/Out Res. Pindefinition
 * D0	RX		PD0		RX			In
 * D1	TX		PD1		TX			In
 * D2	INT0	PD2		US-Echo_L	In
 * D3	INT1	PD3		US-Echo_R	In
 * D4	Digit	PD4		US-Trigger_L Out
 * D5	OC0B	PD5		PWM-B		Out
 * D6	OC0A	PD6		PWM-A		Out
 * D7	Digit	PD7		US-Trigger_R Out	->	DDRD = 0b11110000
 * D8	Digit	PB0		Motor-B		Out
 * D9	Digit	PB1		Motor-A		Out
 *D10	Digit	PB2		LS_LED		Out
 *D11	Unused	PB3					Out
 *D12	Unused	PB4					Out
 *D13	LED		PB5		LED			Out	->	DDRB = 0b11111111
 *A0	Ana		PC0		Line-L3		In
 *A1	Ana		PC1		Line-L2		In
 *A2	Ana		PC2		Line-L1		In
 *A3	Ana		PC3		Line-L0		In
 *A4	Ana		PC4		Line-R0		In
 *A5	Ana		PC5		Line-R1		In	->	DDRC = 0b00000000
 *A6	Ana		ADC6	Line-R2		In
 *A7	Ana		ADC7	Line-R3		In
 */
#define F_CPU 16000000UL //16MHz required for delay
#include <avr/io.h> //Input/Output library
#include <util/delay.h> //Needed for using _delay_...
#include <avr/interrupt.h> //External and internal Interrupts
#include "Init.h"
#include "USART_Functions.h"
#include "Drive_Functions.h"
#include "Interrupt_Service_Routines.h"

//Definitions
#define FOSC				16000000 // Clock Speed
#define BAUD				76800//Baud rate, normally used by Arduino 9600, possible rates: n * 9600
#define MYUBRR				FOSC/16/BAUD-1 //Calculate my baud rate
#define	PWMA				OCR0A //Overflow compare register A
#define PWMB				OCR0B //Overflow compare register B
//#define hundred				0 //First digit of a 3 digit literal
//#define ten					1 //Second digit
//#define one					2 //Third digit
#define Line_analogue_right	Analogue_value[0] //Right line sensor
#define Line_analogue_left	Analogue_value[1] //Left line sensor
#define Distance_1			Analogue_value[2] //IR distance sensor
#define Distance_2			Analogue_value[3] //IR distance sensor

//Macros
#define	LED_ON				PORTB |= (1 << PORTB5) //Push PORTB pin 5 high (LED, pin D13 "Nano")
#define	LED_OFF				PORTB &= ~(1 << PORTB5) //~ Not Operator
#define	LED_TOGGLE			PINB |= (1<<PINB5) //Toggle Arduino Nano pin D13


//Variables
unsigned char z; //For tests

//unsigned int T2ck; //Counter TMR2 overflow
//unsigned char Line_digital_right; //State of right line sensor
//unsigned char Line_digital_middle; //State of middle line sensor
//unsigned char Line_digital_left; //State of left line sensor

//unsigned char US_Time; //Runtime of US signal, HI part
//unsigned char Counter_US; //Counter TMR2 overflow generating a US-trigger every 32ms

//unsigned char Counter_Transmission; //Counter of digits of a transmission of a literal
//unsigned char Counter_Transmission_Data; //Counter of transmitted data

//unsigned char Data[10]; //Number of datas displayed on Data Visualizer
//unsigned char Trans_data[3]; //3 digits who will be transmitted

//unsigned char Channel; //Analogue channel of the ADC
//unsigned char Analogue_value[4]; //Number of analogue inputs

//Funktions, declaration
void Init (void); //Initialize Microcontroller
//void USART_Transmit(unsigned char); //Transmit ASCII to Data Visualizer
void USART_Init(unsigned int); //Initialize USART Transmission/Reception
void Transmit_literal (char); //Transmit literal to Data Visualizer Monitor
void Computing_Transmission_Values (unsigned char); //Compute the 3 digits of a literal
void Data_Visualizer (void); //Define datas to be vizualized

//void Forward (unsigned char, unsigned char); //Direction with velocity A and B,for driving straight and driving a curve
//void Backward (unsigned char, unsigned char); //Direction with velocity A and B,for driving straight and driving a curve
//void Turn_right (unsigned char); //Rotate on the spot right, same velocity of the wheels
//void Turn_left (unsigned char); //Rotate on the spot left, same velocity of the wheels
//void Stop (void); //Stop all Motors


//Main routine
int main(void) {
	Init(); //Initalize the MC
	USART_Init(MYUBRR); //Transmision/reception initialize
	sei(); //Enable all interrupts
	while (0) {
		Data_Visualizer();
	}
	while (1) {
		Data_Visualizer(); //Define the values to be displayed
		if (US_Time > 10) { //No obstacle
			if (Line_digital_left) { //Left light sensor bright?
				if (Line_digital_right) { //Right light sensor bright?
					Forward(128,128); //With half force
				}
				else { //Right light sensor dark
					Turn_right(128); //Turn on the spot
				}
			}
			else { //Left light sensor dark
				if (Line_digital_right) { //Right light sensor bright?
					Turn_left(128); //Turn on the spot with half force
				}
				else { //Both sensors are dark
					Stop(); //For the moment
				}
			}
		}
		else { //So, what is to do, to avoid the obstacle???
			
		}
	}	
}

/***********************************************************************************************/
//Other functions

//Data beeing transmitted to the Data Visualizer
void Data_Visualizer (void) {
	Data[0] = Line_digital_right;
	Data[1] = Line_digital_middle;
	Data[2] = Line_digital_left;
	Data[3] = Line_analogue_right;
	Data[4] = Line_analogue_left;
	//	Data[5] = Color_Value_1;
	//	Data[6] = Color_Value_2;
	Data[7] = Distance_1;
	Data[8] = Distance_2;
	Data[9] = US_Time;
}

/***************************************************************************************/




